c 5.3: modifications for overburden calculation
c 5.5: subroutine weakening for strain and/or strain-rate weakening


* -------------------------------------------------------------------- *
      subroutine bndy(neq,nhbw,nbdy,ibdy,vbdy)
* -------------------------------------------------------------------- *

* subroutine bndy imposes boundary conditions on banded equations

      implicit real*8 (a-h,o-z)
c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      parameter (nrmax=2*nemmax, ncmax=2*nymax+6)
      dimension ibdy(nbdy),vbdy(nbdy)
      common /forces/ndf,s(nrmax,ncmax),sl(nrmax)

      do 300 nb = 1,nbdy
         ie = ibdy(nb)
         sval = vbdy(nb)
         it = nhbw - 1
         i  = ie - nhbw
         do 100 ii=1,it
            i = i + 1
            if(i.ge.1) then
               j = ie - i + 1
               sl(i) = sl(i) - s(i,j)*sval
               s(i,j) = 0.0
            endif
 100     continue
         s(ie,1) = 1.0
         sl(ie) = sval
         i = ie
         do 200 ii=2,nhbw
            i = i + 1
            if(i.gt.neq) goto 200
            sl(i) = sl(i) - s(ie,ii)*sval
            s(ie,ii) = 0.0
 200     continue
 300  continue

      return
      end

* -------------------------------------------------------------------- *
      subroutine centroid (n,cen,xc,grav,option,nx,ny,nem,npe)
* -------------------------------------------------------------------- *
      implicit real *8 (a-h,o-z)

c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      parameter (nrmax=2*nemmax, ncmax=2*nymax+6, nfo=4*(nymax+nxmax+1))

      real*4 rho(nmax)
      integer option

      common /meshval/x(nemmax),y(nemmax)
      common /nodva/nod(nmax,9)
      common /forces/ndf,gstif(nrmax,ncmax),gf(nrmax)
      common /lstif/elstif(18,18),elxy(9,2),bf(9,2),f(18),w(2,9),rho


* calculate top node positions to determine overburden
      icol=((n-1)/ny)+1
      ni = nod(icol*ny, 3)
      nj = nod(icol*ny, 4)
      ymin=0.5*(y(ni)+y(nj))
      cen=0.
      xc=0.

* loop over the nodes
      do 150 i=1,npe
* task 1
         ni = nod(n,i)
         elxy(i,1) = x(ni)
         elxy(i,2) = y(ni)
         cen=cen+y(ni)
         xc=xc+x(ni)
* task 2
         if (option.gt.1) then
            li = (ni-1)*ndf
            do j=1,ndf
               li=li+1
               w(j,i) = gf(li)
            enddo
         endif
* application of body forces - grav includes density scale factor
* task 
         if (option.gt.2) then
            bf(i,1)=0.d0
            bf(i,2)=rho(n)*grav
         endif
 150  continue
      cen=cen/npe - ymin
      xc=xc/npe
      end

*5.3: new centroid returning current column

* -------------------------------------------------------------------- *
      subroutine centroidint (n,icol,cen,xc,grav,option,nx,ny,nem,npe)
* -------------------------------------------------------------------- *
      implicit real *8 (a-h,o-z)

c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      parameter (nrmax=2*nemmax, ncmax=2*nymax+6, nfo=4*(nymax+nxmax+1))

      real*4 rho(nmax)
      integer option

      common /meshval/x(nemmax),y(nemmax)
      common /nodva/nod(nmax,9)
      common /forces/ndf,gstif(nrmax,ncmax),gf(nrmax)
      common /lstif/elstif(18,18),elxy(9,2),bf(9,2),f(18),w(2,9),rho


* calculate top node positions to determine overburden
      icol=((n-1)/ny)+1
      ni = nod(icol*ny, 3)
      nj = nod(icol*ny, 4)
      ymin=0.5*(y(ni)+y(nj))
      cen=0.
      xc=0.

* loop over the nodes
      do 150 i=1,npe
* task 1
         ni = nod(n,i)
         elxy(i,1) = x(ni)
         elxy(i,2) = y(ni)
         cen=cen+y(ni)
         xc=xc+x(ni)
* task 2
         if (option.gt.1) then
            li = (ni-1)*ndf
            do j=1,ndf
               li=li+1
               w(j,i) = gf(li)
            enddo
         endif
* application of body forces - grav includes density scale factor
* task 
         if (option.gt.2) then
            bf(i,1)=0.d0
            bf(i,2)=rho(n)*grav
         endif
 150  continue
      cen=cen/npe - ymin
      xc=xc/npe
      end



* ----------------- Numerical Recipes erfcc -------------------------- *
      FUNCTION ERF(X)
* -------------------------------------------------------------------- *
      implicit real*8(a-h,o-z)
      Z=ABS(X)       
      T=1./(1.+0.5*Z) 
      ERFCC=T*EXP(-Z*Z-1.26551223+T*(1.00002368+T*(.37409196+ 
     *    T*(.09678418+T*(-.18628806+T*(.27886807+T*(-1.13520398+ 
     *    T*(1.48851587+T*(-.82215223+T*.17087277))))))))) 

      IF (X.LT.0.) THEN
        ERF=ERFCC-1.
      ELSE
        ERF=1.-ERFCC
      ENDIF 
      RETURN 
      END 

* -------------------------------------------------------------------- *
      subroutine nodes(nx,ny,iel,npe,nnm)
* -------------------------------------------------------------------- *
* subroutine nodes, assign a "nod" value for each element and each
* node
* caution: the 4 node is set for vertical numerotation, whereas 8 and 9
* nodes are still in horizontal numerotation: use it only if npe=4,iel=1

      Implicit real*8 (a-h,o-z)
c      parameter (nxmax=400,nymax=40,nmax=nxmax*nymax)
      parameter (nxmax=1200,nymax=200,nmax=nxmax*nymax)
      common /nodva/nod(nmax,9)
 
* mesh of quadrilateral elements with 4,8, or 9 nodes

      nex1 = nx + 1
      ney1 = ny + 1
      nxx = iel*nx
      nyy = iel*ny
      nxx1 = nxx + 1
      nyy1 = nyy + 1
      nem = nx*ny
      nnm = nxx1*nyy1 - (iel-1)*nx*ny
      k0 = 0
      
      if(npe.eq.9) then
         nnm = nxx1*nyy1
         k0 = 1
      endif
      
*     generate array nod(i,j)
      
c      nod(1,1) = 1
c      nod(1,2) = iel + 1
c      nod(1,3) = nxx1 + (iel-1)*nex1 + iel + 1
c      nod(1,4) = nod(1,3) - iel

      nod(1,1)=1
      nod(1,2)=nod(1,1)+nyy1
      nod(1,3)=nod(1,2)+1
      nod(1,4)=nod(1,1)+1
      if(npe.eq.4) go to 200
      
      if(npe.eq.9) then
         nod(1,3) = 4*nx + 5
         nod(1,4) = nod(1,3) - iel
      endif
      
      nod(1,5) = 2
      nod(1,6) = nxx1 + npe - 6
      nod(1,7) = nod(1,3) - 1
      nod(1,8) = nxx1 + 1
      if(npe.eq.9) nod(1,9) = nxx1 + 2
        
 200  if(ny.eq.1) go to 230
      
c      m = 1
c      do 220 n=2,ny
c         l = (n-1)*nx + 1
c         do 210 i=1,npe
c            nod(l,i) = nod(m,i) + nxx1 + (iel-1)*nex1 + k0*nx
c 210     continue
c         m = l
c 220  continue

      do 320 n=2,ny
         do 310 i=1,npe
            nod (n,i) = nod(n-1,i)+1
 310     continue
 320  continue

 230  if(nx.eq.1) goto 270
      
c      do 260 ni=2,nx
c         do 240 i=1,npe
c            k1 = iel
c            if(i.eq.6 .or. i.eq.8) k1 = 1 + k0
c            nod(ni,i) = nod(ni-1,i) + k1
c 240     continue
c         m = ni
c        do 260 nj=2,ny
c            l = (nj-1)*nx + ni
c            do 250 j=1,npe
c               nod(l,j) = nod(m,j) + nxx1 + (iel-1)*nex1 + k0*nx
c 250        continue
c            m = l
c 260  continue
      
      mi=1
      do 360 ni=2,nx
         li=(ni-1)*ny+1
         do 340 i=1,npe
            nod(li,i)=nod(mi,i)+nyy1
 340     continue
         mj=li
         do 380 nj=2,ny
            lj=li+nj-1
            do 350 j=1,npe
               nod(lj,j)=nod(mj,j)+1
 350        continue
            mj=lj
 380     continue
         mi=li
 360  continue

 270  continue

      end

* -------------------------------------------------------------------- *
      subroutine shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)
* -------------------------------------------------------------------- *
* subroutine shape evaluates the interpolation functions sf(i)         *
* and their derivatives with respect to the natural coordinates        *
* dsf(i,j), and the derivatives of sf(i) with respect to global        *
* coordinates for four, eight, and nine node rectangular               *
* isoparametric elements.                                              *
*                                                                      *
* sf(i)........interpolation function for node i of the element        *
* dsf(i,j).....derivative of sf(j) w.r.t. xi if i=1  and eta if i=2    *
* gdsf(i,j)....derivative of sf(j) w.r.t. x if i=1 and y if i=2        *
* xnode(i,j)...jth (j=1,2) coordinate of node i of the element         *
* np(i)........array of element nodes (used for defining sf and dsf)   *
* gj(i,j)......jacobian matrix                                         *
* gjinv(i,j)...inverse of the jacobian matrix                          *
* xx,yy........global coordinates corresponding to xi and eta          *
*----------------------------------------------------------------------*

      implicit real*8 (a-h,o-z)
      dimension elxy(9,2),xnode(9,2),np(9),dsf(2,9),gj(2,2),gjinv(2,2),
     *     sf(9),gdsf(2,9)
      data xnode/-1.0d0,2*1.0d0,-1.0d0,0.0d0,1.0d0,0.0d0,-1.0d0,0.0d0,
     *     2*-1.0d0,2*1.0d0,-1.0d0,0.0d0,1.0d0,2*0.0d0/
      data np   /1,2,3,4,5,7,6,8,9/
      fnc(a,b) = a*b

      if(npe-8) 60,10,80

* linear interpolation functions (for four-node element)

 60   continue
      do 70 i=1,npe
         xp = xnode(i,1)
         yp = xnode(i,2)
         xi0  = 1.0 + xi*xp
         eta0 = 1.0 + eta*yp
         sf(i)    = 0.25*fnc(xi0,eta0)
         dsf(1,i) = 0.25*fnc(xp,eta0)
         dsf(2,i) = 0.25*fnc(yp,xi0)
 70   continue
      go to 130

* quadratic interpolation functions (for eight-node element)

 10   continue
      do 40 i=1,npe
         ni = np(i)
         xp = xnode(ni,1)
         yp = xnode(ni,2)
         xi0  = 1.0 + xi*xp
         eta0 = 1.0 + eta*yp
         xi1  = 1.0 - xi*xi
         eta1 = 1.0 - eta*eta
         if(i.le.4) then
            sf(ni)    = 0.25*fnc(xi0,eta0)*(xi*xp+eta*yp-1.0)
            dsf(1,ni) = 0.25*fnc(eta0,xp)*(2.0*xi*xp+eta*yp)
            dsf(2,ni) = 0.25*fnc(xi0,yp)*(2.0*eta*yp+xi*xp)
         else if(i.le.6) then
            sf(ni)    = 0.5*fnc(xi1,eta0)
            dsf(1,ni) = -fnc(xi,eta0)
            dsf(2,ni) = 0.5*fnc(yp,xi1)
         else
            sf(ni)    = 0.5*fnc(eta1,xi0)
            dsf(1,ni) = 0.5*fnc(xp,eta1)
            dsf(2,ni) = -fnc(eta,xi0)
         endif
 40   continue
      go to 130

* quadratic interpolation functions for nine-node element

 80   continue
      do 120 i=1,npe
         ni = np(i)
         xp = xnode(ni,1)
         yp = xnode(ni,2)
         xi0  = 1.0 + xi*xp
         eta0 = 1.0 + eta*yp
         xi1  = 1.0 - xi*xi
         eta1 = 1.0 - eta*eta
         xi2  = xp*xi
         eta2 = yp*eta
         if(i.le.4) then
            sf(ni)    = 0.25*fnc(xi0,eta0)*xi2*eta2
            dsf(1,ni) = 0.25*xp*fnc(eta2,eta0)*(1.0+2.0*xi2)
            dsf(2,ni) = 0.25*yp*fnc(xi2,xi0)*(1.0+2.0*eta2)
         else if(i.le.6) then
            sf(ni)    = 0.5*fnc(xi1,eta0)*eta2
            dsf(1,ni) = -xi*fnc(eta2,eta0)
            dsf(2,ni) = 0.5*fnc(xi1,yp)*(1.0+2.0*eta2)
         else if(i.le.8) then
            sf(ni)    = 0.5*fnc(eta1,xi0)*xi2
            dsf(1,ni) = 0.5*fnc(eta1,xp)*(1.0+2.0*xi2)
            dsf(2,ni) = -eta*fnc(xi2,xi0)
         else
            sf(ni)    = fnc(xi1,eta1)
            dsf(1,ni) = -2.0*xi*eta1
            dsf(2,ni) = -2.0*eta*xi1
         endif
 120  continue

 130  continue
      do 140 i=1,2
         do 140 j=1,2
            gj(i,j) = 0.
            do 140 k=1,npe
               gj(i,j) = gj(i,j) + dsf(i,k)*elxy(k,j)
 140  continue
      det = gj(1,1)*gj(2,2) - gj(1,2)*gj(2,1)
      gjinv(1,1) = gj(2,2)/det
      gjinv(2,2) = gj(1,1)/det
      gjinv(1,2) = -gj(1,2)/det
      gjinv(2,1) = -gj(2,1)/det

      do 150 i=1,2
         do 150 j=1,npe
            gdsf(i,j) = 0.0
            do 150 k=1,2
               gdsf(i,j) = gdsf(i,j) + gjinv(i,k)*dsf(k,j)
 150  continue

      xx = 0.0
      yy = 0.0
      do 160 k=1,npe
         xx = xx + sf(k)*elxy(k,1)
         yy = yy + sf(k)*elxy(k,2)
 160  continue

      return
      end
* -------------------------------------------------------------------- *
      subroutine solve(neqns,nbw,ires)
* -------------------------------------------------------------------- *

* subroutine solve solves a banded symmetric system of linear equations
* in resolving, ires.gt.0, lhs elimination is skipped

      implicit real*8 (a-h,o-z)
c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      parameter (nrmax=2*nemmax, ncmax=2*nymax+6)
      common /forces/ndf,band(nrmax,ncmax),rhs(nrmax)
      
      meqns = neqns - 1
      if(ires.gt.0) goto 90
      do 500 npiv=1,meqns
         npivit = npiv + 1
         lstsub = npiv + nbw - 1
         if(lstsub.gt.neqns)     lstsub = neqns
         do 400 nrow=npivit,lstsub
            
* invert rows and columns for row factor
            ncol = nrow - npiv + 1
            factor = band(npiv,ncol)/band(npiv,1)
            do 200 ncol = nrow,lstsub
               icol = ncol - nrow + 1
               jcol = ncol - npiv + 1
             band(nrow,icol) = band(nrow,icol) - factor*band(npiv,jcol)
 200        continue
            rhs(nrow) = rhs(nrow) - factor*rhs(npiv)
 400     continue
 500  continue
      go to 101

 90   continue
      do 100 npiv=1,meqns
         npivit = npiv + 1
         lstsub = npiv + nbw - 1
         if(lstsub.gt.neqns)     lstsub = neqns
         do 110 nrow=npivit,lstsub
            ncol = nrow - npiv + 1
            factor = band(npiv,ncol)/band(npiv,1)
            rhs(nrow) = rhs(nrow) - factor*rhs(npiv)
 110     continue
 100  continue

 101  continue

* back substitution
      do 800 ijk=2,neqns
         npiv = neqns - ijk + 2
         rhs(npiv) = rhs(npiv)/band(npiv,1)
         lstsub = npiv - nbw + 1
         if(lstsub.lt.1)        lstsub = 1
         npivit = npiv - 1
         do 700 jki=lstsub,npivit
            nrow = npivit - jki + lstsub
            ncol = npiv - nrow + 1
            factor = band(nrow,ncol)
            rhs(nrow) = rhs(nrow) - factor*rhs(npiv)
 700     continue
 800  continue
      rhs(1) = rhs(1)/band(1,1)
      return
      end

* -------------------------------------------------------------------- *
      subroutine stiff(npe,nn,iel,ak1,ak2)
* -------------------------------------------------------------------- *
* stiffness matrix for isoparametric quadrilateral elements

      implicit real*8(a-h,o-z)
c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      real*4 rho(nmax)
      common /lstif/elstif(18,18),elxy(9,2),bf(9,2),f(18),w(2,9),rho
      dimension sf(9),gdsf(2,9),gauss(4,4),wt(4,4),s(9,9),ss(9),
     *          sxy(9,9),sxx(9,9),syy(9,9)
      data gauss/4*0.0d0,-0.577350269189626,0.577350269189626,2*0.0d0,
     *           -.77459667d0,0.0d0,.77459667d0,0.0d0,
     *           -0.861136311594053,-0.339981043584856,
     *           0.339981043584856,0.861136311594053/
      data    wt/2.0d0,3*0.0d0,2*1.0d0,2*0.0d0,0.55555555555556,
     *           0.888888888888889,0.55555555555556,0.0d0,
     *           0.347854845137454,2*0.652145154862546,
     *           0.347854845137454/


      ndf = nn/npe
      ngp = iel+1

*     initialize arrays

      do 20 i=1, npe
c       ss(i) = 0.0     never used
         do 20 j=1, npe
            s(i,j)   = 0.0
            sxx(i,j)  = 0.0
            syy(i,j)  = 0.0
            sxy(i,j) = 0.0
 20   continue
      do 30 i=1,nn
 30      f(i) = 0.0

*     do-loops on numerical (gauss) quadrature begin here

      do 100 ni=1,ngp
         do 100 nj=1,ngp
            xi  = gauss(ni,ngp)
            eta = gauss(nj,ngp)

            call shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)

            const = det*wt(ni,ngp)*wt(nj,ngp)

            do 80 i=1,npe
c       ss(i)   = ss(i)   + const*sf(i)    never used
               do 80 j=1,npe
                  s(i,j)    = s(i,j)    + const*sf(i)*sf(j)
                  sxx(i,j)  = sxx(i,j)  + const*gdsf(1,i)*gdsf(1,j)
                  syy(i,j)  = syy(i,j)  + const*gdsf(2,i)*gdsf(2,j)
                  sxy(i,j)  = sxy(i,j)  + const*gdsf(1,i)*gdsf(2,j)
 80         continue
 100  continue

* element calculations for incompressible fluid flow (penalty method)
*    with full integration on viscous terms and reduced integration
*           on the penalty terms

      ii = 1
      do 350 i=1,npe
         jj=1
         do 340 j=1,npe
            f(ii)   = f(ii)   + bf(j,1)*s(i,j)
            f(ii+1) = f(ii+1) + bf(j,2)*s(i,j)
            elstif(ii,jj)    = ak1*(2.0*sxx(i,j)+syy(i,j))
            elstif(ii,jj+1)  = ak1*sxy(j,i)
            elstif(ii+1,jj)  = ak1*sxy(i,j)
            elstif(ii+1,jj+1)= ak1*(sxx(i,j)+2.0*syy(i,j))
            jj = ndf*j + 1
 340     continue
         ii = ndf*i + 1
 350  continue

* reduced integration on penalty terms

      do 400 ni=1,iel
         do 400 nj=1,iel
            xi  = gauss(ni,iel)
            eta = gauss(nj,iel)

            call shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)

            const = det*wt(ni,iel)*wt(nj,iel)

            ii = 1
            do 380 i=1,npe
               jj = 1
               do 360 j=1,npe
                  elstif(ii,jj)     = elstif(ii,jj) +
     *                 ak2*const*gdsf(1,i)*gdsf(1,j)
                  elstif(ii,jj+1)   = elstif(ii,jj+1) +
     *                 ak2*const*gdsf(1,i)*gdsf(2,j)
                  elstif(ii+1,jj)   = elstif(ii+1,jj) +
     *                 ak2*const*gdsf(2,i)*gdsf(1,j)
                  elstif(ii+1,jj+1) = elstif(ii+1,jj+1) +
     *                 ak2*const*gdsf(2,i)*gdsf(2,j)
                  jj = ndf*j + 1
 360           continue
               ii = ndf*i + 1
 380        continue
 400  continue
      return
      end

* -------------------------------------------------------------------- *
      subroutine strainr (str2inv,e2inv,n,npe,elxy,w)
* -------------------------------------------------------------------- *
* subroutine to compute second invariant of strainrate
*     at the center of quadrilateral elements (iel=1 or 2)

      implicit real*8 (a-h,o-z)
      dimension sf(9),gdsf(2,9),
     *     elxy(9,2),w(2,9)
      
      ux = 0.
      uy = 0.
      vx = 0.
      vy = 0.
      wz = 0.
      
      xi = 0.0
      eta = 0.0
      call shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)
      
      do 110 i=1,npe
         ux = ux + w(1,i)*gdsf(1,i)
         uy = uy + w(1,i)*gdsf(2,i)
         vx = vx + w(2,i)*gdsf(1,i)
         vy = vy + w(2,i)*gdsf(2,i)
c     wz = wz + w(1,i)*sf(i)/xx
 110  continue

      do 111 i=1,npe
         exx = exx + elxy(i,1)*gdsf(1,i)
         exy = exy + elxy(i,1)*gdsf(2,i)
         eyx = eyx + elxy(i,2)*gdsf(1,i)
         eyy = eyy + elxy(i,2)*gdsf(2,i)	
 111  continue
      
      e2inv=dsqrt(ux*ux+(uy+vx)*(uy+vx)/2.d0+vy*vy)
      str2inv=dsqrt(exx*exx+(exy+eyx)*(eyx+exy)/2.d0+eyy*eyy)

*     stress for incompressible stokes flow (penalty method)
c     p   = -ak2*(ux+vy)
c     sx  = 2.0*ak1*ux - p
c     sy  = 2.0*ak1*vy - p
c     sxy = ak1*(uy+vx)
      end

* -------------------------------------------------------------------- *
      subroutine stress(n,npe,elxy,w,ak1,ak2,iel,sxx,sxy)
* -------------------------------------------------------------------- *

* subroutine to compute the gradient of a solution and stresses
*     for quadrilateral elements; 1 given element, not whole model

      implicit real*8 (a-h,o-z)
      integer flag
      dimension sf(9),gdsf(2,9),elxy(9,2),w(2,9)
      common /io/ire,iwr

c     const = 180.0/3.14159265
      ux = 0.
      uy = 0.
      vx = 0.
      vy = 0.
c     wz = 0.

* calculation of stress/gradient of solution at center
*            of a quadrilateral element
      
      xi = 0.0
      eta = 0.0
      
      call shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)
      
      do 110 i=1,npe
         ux = ux + w(1,i)*gdsf(1,i)
         uy = uy + w(1,i)*gdsf(2,i)
         vx = vx + w(2,i)*gdsf(1,i)
         vy = vy + w(2,i)*gdsf(2,i)
c     wz = wz + w(1,i)*sf(i)/xx
 110  continue

* stress for incompressible stokes flow (penalty method)

      p   = -ak2*(ux+vy)
      sx  = 2.0*ak1*ux - p
      sy  = 2.0*ak1*vy - p
      sxy = ak1*(uy+vx)
      sxx = 2.0*ak1*ux
            
* Save in output file
c      write(iwr,4) n,sx,sy,sxy,p,sxx

c 4    format(5x,i5,2x,5(4x,1pe13.6))
      
      end

* -------------------------------------------------------------------- *
      subroutine stressorient(n,npe,elxy,w,ak1,ak2,iel,sxx,syy,fl)
* -------------------------------------------------------------------- *

* subroutine to compute the gradient of a solution and stresses
*     for quadrilateral elements; 1 given element, not whole model

      implicit real *8 (a-h,o-z)
      integer fl
c      real *8 fl
      dimension sf(9),gdsf(2,9),elxy(9,2),w(2,9)
      common /io/ire,iwr
      
c      print *,ak1,ak2
c     const = 180.0/3.14159265
      ux = 0.
      uy = 0.
      vx = 0.
      vy = 0.
c     wz = 0.

* calculation of stress/gradient of solution at center
*            of a quadrilateral element
      
      xi = 0.0
      eta = 0.0
      
      call shape(npe,xi,eta,sf,gdsf,det,elxy,xx,yy)
      
      do 110 i=1,npe
         ux = ux + w(1,i)*gdsf(1,i)
         uy = uy + w(1,i)*gdsf(2,i)
         vx = vx + w(2,i)*gdsf(1,i)
         vy = vy + w(2,i)*gdsf(2,i)
c     wz = wz + w(1,i)*sf(i)/xx
 110  continue

* stress for incompressible stokes flow (penalty method)

      p   = -ak2*(ux+vy)
      sxx  = 2.0*ak1*ux
c - p
      syy  = 2.0*ak1*vy
c - p
      sxy = ak1*(uy+vx)
c      sxx = 2.0*ak1*ux
c      sII=sqrt((sxx+syy)**2+sxy**2)/2
c      alpha=-atan(2*sxy/(sxx-syy))/2
c      if (sxx.lt.syy) alpha=alpha-pi/2
c      s1=(sxx+syy)/2-sII
c      s2=(sxx+syy)/2+sII
c      ord1=int(s1/abs(s1))
c      ord2=int(s2/abs(s2))
            
* Save in output file
c      write(iwr,4) n,alpha,sII,ord1,ord2
       write (iwr,5) n,sxx/1e6,syy/1e6,sxy/1e6,fl
 4    format(5x,i5,2x,2(4x,f12.6),2(2x,i5))
 5    format(5x,i5,2x,3(4x,g18.6),4x,i12)
      
      end


* -------------------------------------------------------------------- *
      subroutine  xygen(nem1,nx1,ny1,nxl,nyl,ire,iloc)
* -------------------------------------------------------------------- *
      implicit real*8 (a-h,o-z)

c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nemmax=nxmx1*nymx1,nmax=nxmax*nymax)
      parameter (pi=3.14159265358979)

      dimension prt(nxmx1)

      common /meshval/x(nemmax),y(nemmax)
* 5.3: new common for overburden 
      common /coverbd/celload(nymax)

* fix the top left corner
      ibase=1
      itop=ny1
      ijun=itop
      xmark=0
      ymark=0
      maxobd=0
      x(ijun)=xmark
      y(ijun)=ymark

* define the first column
      do ilayer=1,nyl
         read (ire,*) nrow,spacing
         do irow=1,nrow
            ymark=ymark+spacing
            ijun=ijun-1
* 5.3: store cell heights  
            celload(ijun)=spacing*1000.d0
            x(ijun)=xmark
            y(ijun)=ymark
         enddo
      enddo


* check
      if (ijun.ne.1) print *,'problem with ygen: ijun final=',ijun

      ijun=ny1+ijun

* loop over "slices": define initial state, and perturbation
      do islice=1,nxl
         nloc=ibase+ny1-iloc-1
         read (ire,*) ncol,spacing,ipert,amp0,para

* calculate the perturbation all pert has to have 0 if icol=0 for continuity
         if (ipert.eq.0) then
*     no perturbation
            do icol=1,ncol
               prt(icol)=0
            enddo
         elseif (ipert.eq.1) then
*     linear pert.
            do icol=1,ncol
               prt(icol)=(icol-0.)/(ncol)
               print *,icol,prt(icol)
            enddo
         elseif (ipert.eq.2) then
*     sine perturb.
            do icol=1,ncol
               prt(icol)=(1-cos(icol*2*pi*spacing/para))/2
            enddo
         else
            stop
         endif

* fill the column
         do icol=1,ncol
            xmark=xmark+spacing
            nth=ibase
* below the perturbation
            do irow=1,ny1-iloc
               amp=amp0*(y(nth)-y(ibase))/(y(nloc)-y(ibase))
               x(ijun)=xmark
c               if (icol.eq.1) then
c                  y(ijun)=y(nth)+prt(icol)*amp
c               else
                  y(ijun)=y(nth)+prt(icol)*amp
c               endif
               ijun=ijun+1
               nth=nth+1
            enddo
* above the perturbation, if necessary
            if (iloc.ne.0) then
               do irow=ny1-iloc+1,ny1
                  amp=amp0*(y(nth)+y(itop))/(y(nloc)-y(itop))
                  x(ijun)=xmark
                  y(ijun)=y(nth)+prt(icol)*amp
                  ijun=ijun+1
                  nth=nth+1
               enddo
            endif
         enddo

* this slice is done
         ibase=ibase+ncol*ny1
         itop=itop+ncol*ny1
         ites=ijun-ny1
         if (ibase.ne.ites) print *,'xygen:ites.ne.ibase, slice',islice
      enddo

      if (itop.ne.nem1) print *,'problem with xgen: itop final=',itop

      end

* -------------------------------------------------------------------- *
      subroutine  weakening(coeff,vmcf,xpass,xmaxi,iwstr,epsilv,epsill,
     &     ws,wsr,sinv)
* -------------------------------------------------------------------- *

	parameter (pi=3.14159265358979)

*	   write(*,*) ws,wsr,sinv,epsill,epsilv,vmcf,xpass,xmaxi,iwstr
           if (iwstr.eq.1) then  
		if (epsilv.lt.epsill) then 
       			coeff=(1-epsilv/epsill*ws)
		else
			if (epsilv.lt.vmcf) then
			coeff=(1-ws)
			else
			coeff=(1-wsr*log10(sinv))*(1-ws)
			endif
		endif
            else
                if (iwstr.eq.2) then        
                   	coeff=1-wsr*log10(sinv)
		else
		   	if (epsilv.lt.epsill) then
				coeff=(1-epsilv/epsill*ws)
			else
				coeff=(1-ws)
			endif
                endif
            endif

*	side taper

	if (xpass.lt.(xmaxi/10.)) then
		coeff=1-(1-coeff)*sin(xpass*pi/(2*xmaxi/10.))
	endif

	if (xpass.gt.(9*xmaxi/10.)) then 
		coeff=1-(1-coeff)*sin((xmaxi-xpass)*pi/(2*xmaxi/10.))
	endif

	end
